@@ -1,5 +1,5 @@
 # ext/horizontal_shard.py
-# Copyright (C) 2005-2018 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -17,10 +17,10 @@ the source distribution.
 
 from .. import inspect
 from .. import util
-from ..orm.session import Session
 from ..orm.query import Query
+from ..orm.session import Session
 
-__all__ = ['ShardedSession', 'ShardedQuery']
+__all__ = ["ShardedSession", "ShardedQuery"]
 
 
 class ShardedQuery(Query):
@@ -43,12 +43,10 @@ class ShardedQuery(Query):
 
     def _execute_and_instances(self, context):
         def iter_for_shard(shard_id):
-            context.attributes['shard_id'] = context.identity_token = shard_id
+            context.attributes["shard_id"] = context.identity_token = shard_id
             result = self._connection_from_session(
-                mapper=self._mapper_zero(),
-                shard_id=shard_id).execute(
-                context.statement,
-                self._params)
+                mapper=self._mapper_zero(), shard_id=shard_id
+            ).execute(context.statement, self._params)
             return self.instances(result, context)
 
         if context.identity_token is not None:
@@ -65,7 +63,13 @@ class ShardedQuery(Query):
             return iter(partial)
 
     def _identity_lookup(
-            self, mapper, primary_key_identity, identity_token=None, **kw):
+        self,
+        mapper,
+        primary_key_identity,
+        identity_token=None,
+        lazy_loaded_from=None,
+        **kw
+    ):
         """override the default Query._identity_lookup method so that we
         search for a given non-token primary key identity across all
         possible identity tokens (e.g. shard ids).
@@ -74,11 +78,15 @@ class ShardedQuery(Query):
 
         if identity_token is not None:
             return super(ShardedQuery, self)._identity_lookup(
-                mapper, primary_key_identity,
-                identity_token=identity_token, **kw
+                mapper,
+                primary_key_identity,
+                identity_token=identity_token,
+                **kw
             )
         else:
             q = self.session.query(mapper)
+            if lazy_loaded_from:
+                q = q._set_lazyload_from(lazy_loaded_from)
             for shard_id in self.id_chooser(q, primary_key_identity):
                 obj = super(ShardedQuery, self)._identity_lookup(
                     mapper, primary_key_identity, identity_token=shard_id, **kw
@@ -88,13 +96,13 @@ class ShardedQuery(Query):
 
             return None
 
-    def _get_impl(
-            self, primary_key_identity, db_load_fn, identity_token=None):
+    def _get_impl(self, primary_key_identity, db_load_fn, identity_token=None):
         """Override the default Query._get_impl() method so that we emit
         a query to the DB for each possible identity token, if we don't
         have one already.
 
         """
+
         def _db_load_fn(query, primary_key_identity):
             # load from the database.  The original db_load_fn will
             # use the given Query object to load from the DB, so our
@@ -117,12 +125,20 @@ class ShardedQuery(Query):
             identity_token = self._shard_id
 
         return super(ShardedQuery, self)._get_impl(
-            primary_key_identity, _db_load_fn, identity_token=identity_token)
+            primary_key_identity, _db_load_fn, identity_token=identity_token
+        )
 
 
 class ShardedSession(Session):
-    def __init__(self, shard_chooser, id_chooser, query_chooser, shards=None,
-                 query_cls=ShardedQuery, **kwargs):
+    def __init__(
+        self,
+        shard_chooser,
+        id_chooser,
+        query_chooser,
+        shards=None,
+        query_cls=ShardedQuery,
+        **kwargs
+    ):
         """Construct a ShardedSession.
 
         :param shard_chooser: A callable which, passed a Mapper, a mapped
@@ -177,16 +193,16 @@ class ShardedSession(Session):
             return self.transaction.connection(mapper, shard_id=shard_id)
         else:
             return self.get_bind(
-                mapper,
-                shard_id=shard_id,
-                instance=instance
+                mapper, shard_id=shard_id, instance=instance
             ).contextual_connect(**kwargs)
 
-    def get_bind(self, mapper, shard_id=None,
-                 instance=None, clause=None, **kw):
+    def get_bind(
+        self, mapper, shard_id=None, instance=None, clause=None, **kw
+    ):
         if shard_id is None:
             shard_id = self._choose_shard_and_assign(
-                mapper, instance, clause=clause)
+                mapper, instance, clause=clause
+            )
         return self.__binds[shard_id]
 
     def bind_shard(self, shard_id, bind):
